==> dal-cdb-2.2.0_dev-20160913.112704-1-sources.jar: com/baijia/tianxiao/dal/util/AreaUtils.java <==
package com.baijia.tianxiao.dal.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.baijia.tianxiao.dal.org.dao.AreaDao;
import com.baijia.tianxiao.dal.org.po.Area;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * Area操作工具类
 * @title AreaUtils
 * @desc TODO 
 * @author zhangbing
 * @date 2016年3月14日
 * @version 1.0
 */
public class AreaUtils {
    
    @Autowired
    private AreaDao areaDao;
    
    public enum AreaLevel {
        COUNTRY(0xFF00000000L), PROVINCE(0xFFFF000000L), CITY(0xFFFFFC0000L), COUNTY(0xFFFFFFFC00L), REGION(
            0xFFFFFFFFFFL), TRANSPORTATION(0xFFFFFFFFFFL);
        private final static AreaLevel[] values = values();
        private Long mask;

        private AreaLevel(Long mask) {
            this.mask = mask;
        }

        public Long getMask() {
            return mask;
        }

        public static AreaLevel valueOf(int pos) {
            if (pos < 0 || pos > values.length) {
                return null;
            }
            return values[pos];
        }
    }

    private static final Map<Integer, Area> cache = new HashMap<Integer, Area>();
    private static final Map<Integer, List<Area>> subCache = Maps.newHashMap();
    private static final Map<String, Map<String, Object>> cascadeCache = Maps.newHashMap();

    public void init() {
        load();
//        cascadeCache();
    }

    private void load() {
        List<Area> areas = this.areaDao.getAll("id", "name", "bname", "level");
        
        List<Area> provinces = Lists.newArrayList();
        Long parentMask = AreaLevel.PROVINCE.mask;
        if (CollectionUtils.isNotEmpty(areas)) {
            for (Area area : areas) {
                cache.put(area.getId(), area);
                if (area.getLevel() == 1) {
                    provinces.add(area);
                } else if (area.getLevel() == 2) {
                    Integer areaId = area.getId();
                    Integer parentId = parentMask.intValue() & areaId;
                    if (subCache.containsKey(parentId)) {
                        subCache.get(parentId).add(area);
                    } else {
                        List<Area> set = Lists.newArrayList();
                        set.add(area);
                        subCache.put(parentId, set);
                    }
                }
            }
        }
    }

    public static String getAreaNameByCode(Integer code_id, AreaLevel level) {
        if (code_id == null) {
            return "";
        }
        if (level == null) {
            Area area = cache.get(code_id);
            if (area == null) {
                return "";
            }
            return area.getName();
        }
        switch (level) {
            case COUNTRY:
            case PROVINCE:
            case CITY:
            case COUNTY:
            case REGION:
            case TRANSPORTATION:
                return getAreaName(code_id, level.getMask().intValue());
            default:
                return getAreaName(code_id, level.getMask().intValue());
        }
    }
    
    public static Map<String, String> getAreaNameByCode(Integer code_id) {
        Map<String, String> map = new HashMap<String, String>();
        Area area = null;
        if(code_id != null) {
            area = cache.get(code_id);
            if(area != null) {
                int level = area.getLevel();
                if(level >= AreaLevel.PROVINCE.ordinal()) {
                    map.put("province", getAreaName(code_id, AreaLevel.PROVINCE.getMask().intValue()));
                }
                if(level >= AreaLevel.CITY.ordinal()) {
                    map.put("city", getAreaName(code_id, AreaLevel.CITY.getMask().intValue()));
                }
                if(level >= AreaLevel.COUNTY.ordinal()) {
                    map.put("county", getAreaName(code_id, AreaLevel.COUNTY.getMask().intValue()));
                }
            }
        }
        return map;
    }

    private static String getAreaName(Integer code_id, Integer level_mask) {
        Area area = cache.get(code_id & level_mask);
        if (area == null) {
            return "";
        }
        return area.getName();
    }

    /**
     * 根据子层级id，获取上层级id
     * 
     * @param sub_code_id
     * @param level_mask 需要获得的层级
     * @return
     */
    public static Integer getAreaIdBySubId(Integer sub_code_id, Integer level_mask) {
        return sub_code_id & level_mask;
    }

    public static String getAreaStrByCode(Integer code_id) {
        if (code_id == null) {
            return "";
        }
        Area area = cache.get(code_id);
        if (area == null) {
            return "";
        }
        int level = area.getLevel();
        if (level < 0) {
            return area.getName();
        }
        if (level > 3) {
            level = 3;
        }
        ArrayList<String> areas = new ArrayList<String>(level + 1);
        String name = null;
        for (int i = 0; i <= level; i++) {
            name = getAreaName(code_id, AreaLevel.valueOf(i).getMask().intValue());
            if (areas.contains(name) == false) {
                areas.add(name);
            }
        }
        name = area.getName();
        if (areas.contains(name) == false) {
            areas.add(name);
        }

        StringBuilder result = new StringBuilder();
        for (String s : areas) {
            result.append("-").append(s);
        }
        result.substring(1);
        return result.toString();
    }

    public static List<Area> getAreaList(Integer key) {
        return subCache.get(key);
    }

    public static String getAreaName(Integer areaId) {
        Area area = cache.get(areaId);
        if (area != null) {
            return area.getBname();
        }
        return "";
    }
    
    public void cascadeCache() {
        List<Area> provinces = this.areaDao.getBylevel(AreaLevel.PROVINCE.ordinal());
        if (CollectionUtils.isNotEmpty(provinces)) {
            for (Area province : provinces) {
                int provinceId = province.getId();
                String provinceName = province.getName();
                
                List<Area> cities = this.areaDao.getBylevel(provinceId, AreaLevel.PROVINCE.mask, AreaLevel.CITY.ordinal());
                
                Map<String, Object> cityCache = Maps.newHashMap();
                Map<String, Object> countyCache = null;
                if (CollectionUtils.isNotEmpty(cities)) {
                    for (Area city :cities) {
                        int cityId = city.getId();
                        String cityName = city.getName();
                        List<Area> counties = this.areaDao.getBylevel(cityId, AreaLevel.CITY.mask, AreaLevel.COUNTY.ordinal());
                        
                        countyCache = Maps.newHashMap();
                        for (Area county : counties) {
                            int countyId = county.getId();
                            String name = county.getName();
                            
                            Map<String, Integer> idCache = Maps.newHashMap();
                            idCache.put("id", countyId);
                            countyCache.put(name, idCache);
                        }
                        cityCache.put(cityName, countyCache);
                        getAreaIdCache(cityId, countyCache);
                    }
                }
                cascadeCache.put(provinceName, cityCache);
                getAreaIdCache(provinceId, cityCache);
            }
        }
    }
    
    public void getAreaIdCache(int id, Map<String, Object> idCache) {
        idCache.put("id", id);
    }
    
    public static Map<String, Map<String, Object>> getCascadeCache() {
        return cascadeCache;
    }
}

==> dal-cdb-2.30.0-20170727.013213-34-sources.jar: com/baijia/tianxiao/dal/util/AreaUtils.java <==
package com.baijia.tianxiao.dal.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.baijia.tianxiao.dal.org.dao.AreaDao;
import com.baijia.tianxiao.dal.org.po.Area;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * Area操作工具类
 * @title AreaUtils
 * @desc TODO 
 * @author zhangbing
 * @date 2016年3月14日
 * @version 1.0
 */
public class AreaUtils {
    
    @Autowired
    private AreaDao areaDao;
    
    public enum AreaLevel {
        COUNTRY(0x3FC0000000L),
        PROVINCE(0xFFFF000000L),
        CITY(0xFFFFFC0000L),
        COUNTY(0xFFFFFFFC00L),
        REGION(0xFFFFFFFFFFL),
        TRANSPORTATION(0xFFFFFFFFFFL);

        private final static AreaLevel[] values = values();
        private Long mask;

        private AreaLevel(Long mask) {
            this.mask = mask;
        }

        public Long getMask() {
            return mask;
        }

        public static AreaLevel valueOf(int pos) {
            if (pos < 0 || pos > values.length) {
                return null;
            }
            return values[pos];
        }
    }

    private static final Map<Long, Area> cache = new HashMap<Long, Area>();
    private static final Map<Long, List<Area>> subCache = Maps.newHashMap();
    private static final Map<String, Map<String, Object>> cascadeCache = Maps.newHashMap();

    public void init() {
        load();
//        cascadeCache();
    }

    private void load() {
        List<Area> areas = this.areaDao.getAll("id", "name", "bname", "level");
        
        List<Area> provinces = Lists.newArrayList();
        Long parentMask = AreaLevel.PROVINCE.mask;
        if (CollectionUtils.isNotEmpty(areas)) {
            for (Area area : areas) {
                cache.put(area.getId(), area);
                if (area.getLevel() == 1) {
                    provinces.add(area);
                } else if (area.getLevel() == 2) {
                    Long areaId = area.getId();
                    Long parentId = parentMask & areaId;
                    if (subCache.containsKey(parentId)) {
                        subCache.get(parentId).add(area);
                    } else {
                        List<Area> set = Lists.newArrayList();
                        set.add(area);
                        subCache.put(parentId, set);
                    }
                }
            }
        }
    }

    public static String getAreaNameByCode(Long code_id, AreaLevel level) {
        if (code_id == null) {
            return "";
        }
        if (level == null) {
            Area area = cache.get(code_id);
            if (area == null) {
                return "";
            }
            return area.getName();
        }
        switch (level) {
            case COUNTRY:
            case PROVINCE:
            case CITY:
            case COUNTY:
            case REGION:
            case TRANSPORTATION:
                return getAreaName(code_id, level.getMask().intValue());
            default:
                return getAreaName(code_id, level.getMask().intValue());
        }
    }
    
    public static Map<String, String> getAreaNameByCode(Long code_id) {
        Map<String, String> map = new HashMap<String, String>();
        Area area = null;
        if(code_id != null) {
            area = cache.get(code_id);
            if(area != null) {
                int level = area.getLevel();
                if(level >= AreaLevel.PROVINCE.ordinal()) {
                    map.put("province", getAreaName(code_id, AreaLevel.PROVINCE.getMask().intValue()));
                }
                if(level >= AreaLevel.CITY.ordinal()) {
                    map.put("city", getAreaName(code_id, AreaLevel.CITY.getMask().intValue()));
                }
                if(level >= AreaLevel.COUNTY.ordinal()) {
                    map.put("county", getAreaName(code_id, AreaLevel.COUNTY.getMask().intValue()));
                }
            }
        }
        return map;
    }

    private static String getAreaName(Long code_id, Integer level_mask) {
        Area area = cache.get(code_id & level_mask);
        if (area == null) {
            return "";
        }
        return area.getName();
    }

    /**
     * 根据子层级id，获取上层级id
     * 
     * @param sub_code_id
     * @param level_mask 需要获得的层级
     * @return
     */
    public static Long getAreaIdBySubId(Long sub_code_id, Integer level_mask) {
        return sub_code_id & level_mask;
    }

    public static String getAreaStrByCode(Long code_id) {
        if (code_id == null) {
            return "";
        }
        Area area = cache.get(code_id);
        if (area == null) {
            return "";
        }
        int level = area.getLevel();
        if (level < 0) {
            return area.getName();
        }
        if (level > 3) {
            level = 3;
        }
        ArrayList<String> areas = new ArrayList<String>(level + 1);
        String name = null;
        for (int i = 0; i <= level; i++) {
            name = getAreaName(code_id, AreaLevel.valueOf(i).getMask().intValue());
            if (areas.contains(name) == false) {
                areas.add(name);
            }
        }
        name = area.getName();
        if (areas.contains(name) == false) {
            areas.add(name);
        }

        StringBuilder result = new StringBuilder();
        for (String s : areas) {
            result.append("-").append(s);
        }
        result.substring(1);
        return result.toString();
    }

    public static List<Area> getAreaList(Integer key) {
        return subCache.get(key);
    }

    public static String getAreaName(Integer areaId) {
        Area area = cache.get(areaId);
        if (area != null) {
            return area.getBname();
        }
        return "";
    }
    
    public void cascadeCache() {
        List<Area> provinces = this.areaDao.getBylevel(AreaLevel.PROVINCE.ordinal());
        if (CollectionUtils.isNotEmpty(provinces)) {
            for (Area province : provinces) {
                long provinceId = province.getId();
                String provinceName = province.getName();
                
                List<Area> cities = this.areaDao.getBylevel(provinceId, AreaLevel.PROVINCE.mask, AreaLevel.CITY.ordinal());
                
                Map<String, Object> cityCache = Maps.newHashMap();
                Map<String, Object> countyCache = null;
                if (CollectionUtils.isNotEmpty(cities)) {
                    for (Area city :cities) {
                        long cityId = city.getId();
                        String cityName = city.getName();
                        List<Area> counties = this.areaDao.getBylevel(cityId, AreaLevel.CITY.mask, AreaLevel.COUNTY.ordinal());
                        
                        countyCache = Maps.newHashMap();
                        for (Area county : counties) {
                            long countyId = county.getId();
                            String name = county.getName();
                            
                            Map<String, Long> idCache = Maps.newHashMap();
                            idCache.put("id", countyId);
                            countyCache.put(name, idCache);
                        }
                        cityCache.put(cityName, countyCache);
                        getAreaIdCache(cityId, countyCache);
                    }
                }
                cascadeCache.put(provinceName, cityCache);
                getAreaIdCache(provinceId, cityCache);
            }
        }
    }
    
    public void getAreaIdCache(long id, Map<String, Object> idCache) {
        idCache.put("id", id);
    }
    
    public static Map<String, Map<String, Object>> getCascadeCache() {
        return cascadeCache;
    }
}
